@@ -35,7 +35,8 @@ def getGHOST(real=False, verbose=False):
     if not os.path.exists(install_path + '/database'):
         os.makedirs(install_path + '/database')
     if real:
-        url = 'http://ghost.ncsa.illinois.edu/static/database/GHOST.csv'
+        #url = 'http://ghost.ncsa.illinois.edu/static/database/GHOST.csv'
+        url = 'https://www.dropbox.com/s/esytx5xpyegm1it/GHOST.csv?dl=1'
         r = requests.get(url)
         fname = install_path + '/database/GHOST.csv'
         open(fname , 'wb').write(r.content)
