@@ -52,7 +52,8 @@ def build_sfd_dir(fname='./sfddata-master.tar.gz'):
     return
 
 def get_photoz_weights(fname='./MLP_lupton.hdf5'):
-    url = 'http://ghost.ncsa.illinois.edu/static/MLP_lupton.hdf5'
+    #url = 'http://ghost.ncsa.illinois.edu/static/MLP_lupton.hdf5'
+    url = 'https://www.dropbox.com/s/7bim5sssnw9w6pg/MLP_lupton.hdf5?dl=1'
     response = requests.get(url, stream=True)
     if response.status_code == 200:
         with open(fname, 'wb') as f:
